package aktbus.core;

import java.io.*;
import java.util.*;
import java.net.*;
import sunlabs.brazil.util.http.*;
import java.lang.reflect.Constructor;

// Jena packages
import com.hp.hpl.mesa.rdf.jena.mem.ModelMem;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.*;
import com.hp.hpl.mesa.rdf.jena.common.*;

public class RDFAktMessage extends ModelMem implements AktMessageInterface {
    
    static String HTTP_VERSION="1.1";	//HTTP version string
    static final int BUFFER_SIZE=10240;
    
    // CONSTRUCTORS
    //--------------
    
    public RDFAktMessage() throws RDFException, Exception { 
	this("", "", "");
    }

    public RDFAktMessage(String message_to_parse) throws RDFException, Exception {
	StringReader msgReader = new StringReader(message_to_parse);
	this.read(msgReader, "");
    }	
       
    public RDFAktMessage(URL sender,URL receiver,String simple_content) throws RDFException, Exception {
	this(sender.toString(), receiver.toString(), simple_content);
    }
    
    public RDFAktMessage(String sender,String receiver,String simple_content) throws RDFException, Exception {

	// Create an empty model
	super();

	Calendar now;
	String date_string, time_string;
	int year, month, day;
	int hour, min, sec;
	String id_string, something;
	

	Resource container = this.createResource(AKTVOCAB.Container);
	Resource header = this.createResource(AKTVOCAB.SimpleHeader);
	Resource content = this.createResource(AKTVOCAB.SimpleContent);
	container.addProperty(AKTVOCAB.hasHeader, header);
	container.addProperty(AKTVOCAB.hasContent, content);
	// Add sender and receiver properties to header
	header.addProperty(AKTVOCAB.hasSenderURI, sender);
	header.addProperty(AKTVOCAB.hasReceiverURI, receiver);
	// Add simple message property to content
	content.addProperty(AKTVOCAB.hasSimpleMessage, simple_content);
	// Create time and date...
	now = Calendar.getInstance(); //create current time object
	year = now.get(Calendar.YEAR); //get date & time as integers
	month = map_month(now.get(Calendar.MONTH));			
	day = now.get(Calendar.DAY_OF_MONTH);
	hour = now.get(Calendar.HOUR_OF_DAY);
	min = now.get(Calendar.MINUTE);
	sec = now.get(Calendar.SECOND);
	date_string = year + "-" + month + "-" + day; //construct date & time strings
	time_string = hour + ":" + min + ":" + sec;
	// Add time and date to message...
	header.addProperty(AKTVOCAB.hasDateString, date_string);
	header.addProperty(AKTVOCAB.hasTimeString, time_string);

	// Create id_string and add to message
	something = "Something";
	id_string = sender.toString() + "/" + date_string + "/" + time_string + "/" + something;
	container.addProperty(AKTVOCAB.id, id_string);
    }
	

    // METHODS TO MANIPULATE MESSAGE
    //------------------------------
    
    public String get_receiver() {
	try {
	    Resource header = get_header();
	    return header.getProperty(AKTVOCAB.hasReceiverURI).getString();
	}
	catch (RDFException e) {
	    return null;
	}
    }
    
    public String get_sender() {
	try {	  
	    Resource header = get_header();
	    return header.getProperty(AKTVOCAB.hasSenderURI).getString();
	}
	catch (RDFException e) {
	    return null;
	}
    }
    
    public String get_simple_message() {
	try {	   
	    Resource content = get_content();
	    return content.getProperty(AKTVOCAB.hasSimpleMessage).getString();
	}
	catch (RDFException e) {
	    return null;
	}
    }

    public Resource get_container() throws RDFException {
	ResIterator iter = this.listSubjectsWithProperty(RDF.type, AKTVOCAB.Container);
	Resource container = iter.next();
	iter.close();
	return container;
    }

    public Resource get_header() throws RDFException {
	Resource container = get_container();
	Resource header = container.getProperty(AKTVOCAB.hasHeader).getResource();
	return header;
    }

    public Resource get_content() throws RDFException {
	Resource container = get_container();
	Resource content = container.getProperty(AKTVOCAB.hasContent).getResource();
	return content;
    }

    public void set_receiver(java.lang.String receiver) {
	try {	  
	    Resource header = get_header();	   
	    Statement oldStatement = header.getProperty(AKTVOCAB.hasReceiverURI);
	    oldStatement.set(receiver);	  
	}
	catch (RDFException e) {
	System.out.println(e.toString()); 
	}
    }

    public void set_receiver(java.net.URL receiver) {
	set_receiver(receiver.toString());
    }

    public void set_sender(java.lang.String sender) {
	try {	  
	    Resource header = get_header();	   
	    Statement oldStatement = header.getProperty(AKTVOCAB.hasSenderURI);
	    oldStatement.set(sender);	  
	}
	catch (RDFException e) {
	System.out.println(e.toString()); 
	}
}
    public void set_sender(java.net.URL sender) {
	set_sender(sender.toString());
    }

    public void set_simple_message(java.lang.String message) {
	try {	  
	    Resource content = get_content();	   
	    Statement oldStatement = content.getProperty(AKTVOCAB.hasSimpleMessage);
	    oldStatement.set(message);	  
	}
	catch (RDFException e) {
	System.out.println(e.toString()); 
	}
}
    
    // METHODS TO SEND MESSAGE
    //-------------------------
    // (COPIED FROM ORIGINAL AKTBUS CODE)
    /**
     * Send an <em>AktMessage</em> with blocking, using specified receiver URI in the <em>AktMessage</em> object.
     *
     * @throws	java.lang.Exception		When a general error occurs.
     * @throws	java.io.IOException		When I/O-related error occurs.
     * @throws	java.io.InterruptedIOException	When a timeout occurs.
     *
     */
    public void send()
	throws	java.lang.Exception,
		java.io.IOException,
		java.io.InterruptedIOException
    {
	send(0);
    } //end method send
    
    
    /**
     * Send to the receiver as specified in the <em>AktMessage</em>, with timeout.
     * A receiver must be specified in the <em>AktMessage</em>.
     *
     * @param timeout The timeout (in second) on the connection to the receiver.
     *
     * @throws	java.lang.Exception		When a general error occurs.
     * @throws	java.io.IOException		When I/O-related error occurs.
     * @throws	java.io.InterruptedIOException	When a timeout occurs.
     */
    public void send(int timeout)
	throws	java.lang.Exception,
		java.io.IOException,
		java.io.InterruptedIOException
    {
	URL receiver_uri;
	
	//
	// when you "send", the sender URI must be specified
	// or you should use send_and_receive
	//
	//if (get_sender()==null)				//check if sender URI is specified
	//	throw new Exception("No sender URI specified. Better use send_and_receive.");
	receiver_uri=new URL(get_receiver());		//get receiver URI
	send(receiver_uri, timeout);			//call the send method
    } //end method send
    
    /**
     * Sends an <em>AktMessage</em> to a certain recipient.
     * A receiver URI must be specified in the <em>AktMessage</em>, although the two do not
     * have to be the same.
     *
     * @param	sender_uri	The URL of the sender. A <code>null</code> means piggy-packed reply.
     * @param	receiver_uri	The URL of the receiver.
     * @param	timeout		The timeout value waiting for a response from the other side, in second.
     *
     * @throws	java.lang.Exception		When sending fails. i.e. reply is not OK.
     * @throws	java.io.IOException		When I/O-related error occurs.
     * @throws	java.io.InterruptedIOException	When a timeout occurs.
     */
    public void send(URL receiver_uri,int timeout)
	throws	java.lang.Exception,
		java.io.IOException,
		java.io.InterruptedIOException
    {
	PrintWriter out;
	BufferedReader in;
	StringWriter sw;
	HttpRequest req;
	
	//
	// make sure that sender URI is specified in the AktMessage
	//
	//if (get_sender()==null)
	//	throw new Exception("No sender URI specified. Better use send_and_receive.");
	
	//
	// make sure that receiver URI is specified
	//
	if (get_receiver()==null)
	    throw new Exception("Receiver must be specified for an AktMessage to be sent.");
	
	//
	// put message into a String
	//
	sw=new StringWriter();				//create string writer for XML message
	ugly_write(sw);					//ugl-write XML to buffer
	sw.close();					//close string writer
	
	//
	// Sun's Brazil doesn't put "/" in the HTTP header if it is missing in the end of the URL
	// have to put it back by ourselves
	//
	if (receiver_uri.getFile().length()==0)						//see if file part of URL is empty
	    req=new HttpRequest(receiver_uri.toString()+"/");			//if yes, put in "/"
	else req=new HttpRequest(receiver_uri);
	
	//
	// set parameter values, output POST request
	//
	req.setMethod("POST");								//set HTTP method to POST
	req.setRequestHeader("Content-Type", "text/xml");				//set content-type
	out=new PrintWriter(req.getOutputStream());					//create output stream
	out.write(sw.toString());							//write string
	out.close();									//close output stream
	
	req.connect();									//now connect to server
	
	//
	// close input streams, as no piggy-packed reply is expected
	//
	in = new BufferedReader(new InputStreamReader(req.getInputStream()));		//create input stream
	in.close();									//close input stream
	req.disconnect();								//disconnect from host
	if (req.getResponseCode()!=200)							//examine reponse code
	    throw new Exception(req.status);
    } //end method send
    
    
    // METHODS TO PRINT MESSAGE
    //-------------------------
    
    public void ugly_write(java.io.Writer out) {
	AktPrettyWriter pretty = new AktPrettyWriter();
	pretty.writeMessage(this, out);
    }  

    public void pretty_write(java.io.Writer out) {
	AktPrettyWriter pretty = new AktPrettyWriter();
	pretty.prettyWriteMessage(this, out);
    }  

    /**
     * FROM ORIGINAL AKTBUS CODE
     * Maps Calendar's month value into 1-12
     *
     * @param calendar_month_value	The integer value from Calendar.MONTH.
     * @return	An integer 1-12.
     */
    protected int map_month(int calendar_month_value)
	throws java.lang.Exception
    {
	int i;
	int map[]={
	    Calendar.JANUARY,
	    Calendar.FEBRUARY,
	    Calendar.MARCH,
	    Calendar.APRIL,
	    Calendar.MAY,
	    Calendar.JUNE,
	    Calendar.JULY,
	    Calendar.AUGUST,
	    Calendar.SEPTEMBER,
	    Calendar.OCTOBER,
	    Calendar.NOVEMBER,
	    Calendar.DECEMBER
	};
	
	for (i=0;i<map.length;i++)
	    {
		if (map[i]==calendar_month_value)
		    return(i+1);
	    }
	throw new Exception("Illegal calendar month value");
    }	//end method map_month
}
